@@ -1,5 +1,5 @@
 from .function import (
-    log, exp, relu, sigmoid, softmax, tanh,
+    abs, max, min, log, exp, relu, sigmoid, softmax, tanh,
 )
 from .random import (
     normal, normal_like,
@@ -10,5 +10,5 @@ from .tensor import (
     zeros, zeros_like, ones, ones_like,
     add, sub, neg, mul, truediv, matmul, power, dot,
     sum, mean,
-    transpose, permute, concat
+    transpose, permute, concat, reshape, flatten, pad
 )
