@@ -7,18 +7,22 @@ import numpy as np
 from typing import (
     List,
     Tuple,
-    NamedTuple,
+    Union,
+    Sequence,
     Callable,
     Optional,
-    Union
+    NamedTuple,
+    Dict,
 )
 
 
 class Dependency(NamedTuple):
     tensor: Tensor
-    grad_fn: Callable[[np.ndarray], np.ndarray]
+    grad_fn: Callable[[np.ndarray, Optional[Dict]], np.ndarray]
+    meta: Dict = None
 
 
+Shapable = Union[Tuple, List, int]
 Arrayable = Union[float, list, np.ndarray]
 Tensorable = Union['Tensor', float, np.ndarray]
 
@@ -55,7 +59,7 @@ class Tensor:
     @data.setter
     def data(self, new: np.ndarray) -> None:
         self._data = new
-        # detach gradient if set new data
+        # detach gradient if set new datasets
         self.grad = None
 
     def numpy(self) -> np.ndarray:
@@ -70,29 +74,29 @@ class Tensor:
 
         if grad is None:
             if self.shape == ():
-                grad = Tensor(1)
+                grad = Tensor(1.)
             else:
                 raise RuntimeError(
                     "grad must be specified for non-zero-tensor")
         self.grad.data += grad.data
 
         for dependency in self.dependencies:
-            backward_grad = dependency.grad_fn(grad.data)
+            backward_grad = dependency.grad_fn(grad.data, dependency.meta)
             dependency.tensor.backward(Tensor(backward_grad))
 
     def dim(self) -> int:
-        """Number of self.data dimensions"""
-        return self._data.ndim()
+        """Number of self.datasets dimensions"""
+        return self._data.ndim
 
     def numel(self) -> int:
-        """the product of the self.data’s dimensions."""
+        """the product of the self.datasets’s dimensions."""
         return self._data.size
 
-    def sum(self) -> Tensor:
-        return sum(self)
+    def sum(self, axis: Optional[Shapable] = None) -> Tensor:
+        return sum(self, axis)
 
-    def mean(self) -> Tensor:
-        return mean(self)
+    def mean(self, axis: Optional[Shapable] = None) -> Tensor:
+        return mean(self, axis)
 
     def transpose(self, dim1: int, dim2: int) -> Tensor:
         return transpose(self, dim1, dim2)
@@ -100,6 +104,12 @@ class Tensor:
     def permute(self, axes: Union[List, Tuple]) -> Tensor:
         return permute(self, axes)
 
+    def reshape(self, *sp):
+        return reshape(self, sp)
+
+    def flatten(self, start_dim=0, end_dim=-1):
+        return flatten(self, start_dim, end_dim)
+
     @property
     def T(self) -> Tensor:
         return permute(self, list(range(len(self.shape) - 1, -1, -1)))
@@ -213,53 +223,139 @@ def ones_like(a: Tensor, dtype=None, requires_grad=False) -> Tensor:
     return Tensor(np.ones_like(a=a, dtype=dtype, shape=a.shape), requires_grad=requires_grad)
 
 
-def sum(t: Tensor) -> Tensor:
-    # TODO: test, support axis param
-    """
-    Takes a tensor and return the sum of its components
-    """
-    data = t.data.sum()
-    requires_grad = t.requires_grad
-    dependencies: List[Dependency] = []
-
-    if requires_grad:
-        def grad_fn(grad: np.ndarray) -> np.ndarray:
-            return grad * np.ones_like(t.data)
-
-        dependencies.append(Dependency(t, grad_fn))
-
-    return Tensor(data, requires_grad, dependencies)
-
-
-def mean(t: Tensor) -> Tensor:
-    # TODO: test, support axis param
-    return t.sum() / t.numel()
-
-
 def _slice(t: Tensor, idxs) -> Tensor:
-    # TODO: test
     data = t.data[idxs]
     requires_grad = t.requires_grad
     dependencies: List[Dependency] = []
 
     if requires_grad:
-        def grad_fn(grad: np.ndarray) -> np.ndarray:
-            bigger_grad = np.zeros_like(data)
-            bigger_grad[idxs] = grad
-            return bigger_grad
+        def grad_fn(grad: np.ndarray, _) -> np.ndarray:
+            ret = np.zeros_like(t.data)
+            ret[idxs] = grad
+            assert ret.shape == t.shape
+            return ret
 
-        dependencies.append(Dependency(t, grad_fn))
+        dependencies.append(Dependency(t, grad_fn, meta={"name": "slice"}))
 
     return Tensor(data, requires_grad, dependencies)
 
 
+def pad(t: Tensor, pad_width, mode='constant', **kwargs) -> Tensor:
+    data = np.pad(t.data, pad_width, mode, **kwargs)
+    requires_grad = t.requires_grad
+    dependencies: List[Dependency] = []
+
+    if t.requires_grad:
+        def grad_fn(grad: np.ndarray, _) -> np.ndarray:
+            idx = tuple([np.s_[pad_width[i][0]:grad.shape[i] - pad_width[i][1]] for i in range(grad.ndim)])
+            return grad[idx]
+
+        dependencies.append(Dependency(t, grad_fn, meta={"name": "pad"}))
+
+    return Tensor(data, requires_grad, dependencies)
+
+
+def reshape(t: Tensor, shape) -> Tensor:
+    data = t.data.reshape(shape)
+    requires_grad = t.requires_grad
+    dependencies: List[Dependency] = []
+
+    if t.requires_grad:
+        def grad_fn(grad: np.ndarray, _) -> np.ndarray:
+            return grad.reshape(t.shape)
+
+        dependencies.append(Dependency(t, grad_fn, meta={"name": "reshape"}))
+
+    return Tensor(data, requires_grad, dependencies)
+
+
+def flatten(t: Tensor, start_dim=0, end_dim=-1) -> Tensor:
+    if start_dim < 0:
+        start_dim = t.data.ndim + start_dim
+    if end_dim < 0:
+        end_dim = t.data.ndim + end_dim
+    sp = []
+    for i in range(0, start_dim):
+        sp.append(t.shape[i])
+    p = 1
+    for i in range(start_dim, end_dim + 1):
+        p *= t.shape[i]
+    sp.append(p)
+    for i in range(end_dim + 1, t.data.ndim):
+        sp.append(t.shape[i])
+    data = t.data.reshape(sp)
+    requires_grad = t.requires_grad
+    dependencies: List[Dependency] = []
+
+    if t.requires_grad:
+        def grad_fn(grad: np.ndarray, _) -> np.ndarray:
+            return grad.reshape(t.shape)
+
+        dependencies.append(Dependency(t, grad_fn, meta={"name": "flatten"}))
+
+    return Tensor(data, requires_grad, dependencies)
+
+
+def concat(tensors: Sequence[Tensor], axis: int = 0) -> Tensor:
+    assert len(tensors) > 0
+    dim = tensors[0].dim()
+    data = np.concatenate([t.data for t in tensors], axis=axis)
+    requires_grad = True in [t.requires_grad for t in tensors]
+    dependencies: List[Dependency] = []
+
+    a = 0
+    b = 0
+    for i, t in enumerate(tensors):
+        b += tensors[i].shape[axis]
+        if t.requires_grad:
+            def grad_fn(grad: np.ndarray, meta) -> np.ndarray:
+                idx = tuple([np.s_[meta["a"]:meta["b"]] if j == axis else np.s_[:] for j in range(dim)])
+                return grad[idx]
+
+            dependencies.append(Dependency(t, grad_fn, meta={"name": "concat", "id": i, "a": a, "b": b}))
+        a += tensors[i].shape[axis]
+
+    return Tensor(data, requires_grad, dependencies)
+
+
+def sum(t: Tensor, axis: Optional[Shapable] = None) -> Tensor:
+    """
+    Takes a tensor and return the sum of its components
+    """
+    data = t.data.sum(axis=axis)
+    requires_grad = t.requires_grad
+    dependencies: List[Dependency] = []
+
+    if requires_grad:
+        if axis is None:
+            def grad_fn(grad: np.ndarray, _) -> np.ndarray:
+                return grad * np.ones_like(t.data)
+        else:
+            def grad_fn(grad: np.ndarray, _) -> np.ndarray:
+                return np.expand_dims(grad, axis=axis) * np.ones_like(t.data)
+
+        dependencies.append(Dependency(t, grad_fn, meta={"name": "sum"}))
+
+    return Tensor(data, requires_grad, dependencies)
+
+
+def mean(t: Tensor, axis: Optional[Shapable] = None) -> Tensor:
+    data = t.sum(axis=axis)
+    if axis is None:
+        numel = t.numel()
+    else:
+        total_numel = t.numel()
+        numel = total_numel / data.numel()
+    return data / numel
+
+
 def add(t1: Tensor, t2: Tensor) -> Tensor:
     data = np.add(t1.data, t2.data)
     requires_grad = t1.requires_grad or t2.requires_grad
     dependencies: List[Dependency] = []
 
     if t1.requires_grad:
-        def grad_fn1(grad: np.ndarray) -> np.ndarray:
+        def grad_fn1(grad: np.ndarray, _) -> np.ndarray:
             ndims_added = grad.ndim - t1.data.ndim
             for _ in range(ndims_added):
                 grad = grad.sum(axis=0)
@@ -268,10 +364,10 @@ def add(t1: Tensor, t2: Tensor) -> Tensor:
                     grad = grad.sum(axis=i, keepdims=True)
             return grad
 
-        dependencies.append(Dependency(t1, grad_fn1))
+        dependencies.append(Dependency(t1, grad_fn1, meta={"name": "add_lhs"}))
 
     if t2.requires_grad:
-        def grad_fn2(grad: np.ndarray) -> np.ndarray:
+        def grad_fn2(grad: np.ndarray, _) -> np.ndarray:
             ndims_added = grad.ndim - t2.data.ndim
             for _ in range(ndims_added):
                 grad = grad.sum(axis=0)
@@ -280,7 +376,7 @@ def add(t1: Tensor, t2: Tensor) -> Tensor:
                     grad = grad.sum(axis=i, keepdims=True)
             return grad
 
-        dependencies.append(Dependency(t2, grad_fn2))
+        dependencies.append(Dependency(t2, grad_fn2, meta={"name": "add_rhs"}))
 
     return Tensor(data, requires_grad, dependencies)
 
@@ -291,7 +387,7 @@ def sub(t1: Tensor, t2: Tensor) -> Tensor:
     dependencies: List[Dependency] = []
 
     if t1.requires_grad:
-        def grad_fn1(grad: np.ndarray) -> np.ndarray:
+        def grad_fn1(grad: np.ndarray, _) -> np.ndarray:
             ndims_added = grad.ndim - t1.data.ndim
             for _ in range(ndims_added):
                 grad = grad.sum(axis=0)
@@ -300,10 +396,10 @@ def sub(t1: Tensor, t2: Tensor) -> Tensor:
                     grad = grad.sum(axis=i, keepdims=True)
             return grad
 
-        dependencies.append(Dependency(t1, grad_fn1))
+        dependencies.append(Dependency(t1, grad_fn1, meta={"name": "sub_lhs"}))
 
     if t2.requires_grad:
-        def grad_fn2(grad: np.ndarray) -> np.ndarray:
+        def grad_fn2(grad: np.ndarray, _) -> np.ndarray:
             ndims_added = grad.ndim - t2.data.ndim
             for _ in range(ndims_added):
                 grad = grad.sum(axis=0)
@@ -312,7 +408,7 @@ def sub(t1: Tensor, t2: Tensor) -> Tensor:
                     grad = grad.sum(axis=i, keepdims=True)
             return -grad
 
-        dependencies.append(Dependency(t2, grad_fn2))
+        dependencies.append(Dependency(t2, grad_fn2, meta={"name": "sub_rhs"}))
 
     return Tensor(data, requires_grad, dependencies)
 
@@ -323,7 +419,7 @@ def neg(t: Tensor) -> Tensor:
     dependencies: List[Dependency] = []
 
     if requires_grad:
-        dependencies.append(Dependency(t, lambda x: -x))
+        dependencies.append(Dependency(t, lambda x, _: -x, meta={"name": "neg"}))
 
     return Tensor(data, requires_grad, dependencies)
 
@@ -334,7 +430,7 @@ def mul(t1: Tensor, t2: Tensor) -> Tensor:
     dependencies: List[Dependency] = []
 
     if t1.requires_grad:
-        def grad_fn1(grad: np.ndarray) -> np.ndarray:
+        def grad_fn1(grad: np.ndarray, _) -> np.ndarray:
             grad = grad * t2.data
             ndims_added = grad.ndim - t1.data.ndim
             for _ in range(ndims_added):
@@ -344,10 +440,10 @@ def mul(t1: Tensor, t2: Tensor) -> Tensor:
                     grad = grad.sum(axis=i, keepdims=True)
             return grad
 
-        dependencies.append(Dependency(t1, grad_fn1))
+        dependencies.append(Dependency(t1, grad_fn1, meta={"name": "mul_lhs"}))
 
     if t2.requires_grad:
-        def grad_fn2(grad: np.ndarray) -> np.ndarray:
+        def grad_fn2(grad: np.ndarray, _) -> np.ndarray:
             grad = grad * t1.data
             ndims_added = grad.ndim - t2.data.ndim
             for _ in range(ndims_added):
@@ -357,7 +453,7 @@ def mul(t1: Tensor, t2: Tensor) -> Tensor:
                     grad = grad.sum(axis=i, keepdims=True)
             return grad
 
-        dependencies.append(Dependency(t2, grad_fn2))
+        dependencies.append(Dependency(t2, grad_fn2, meta={"name": "mul_rhs"}))
 
     return Tensor(data, requires_grad, dependencies)
 
@@ -369,7 +465,7 @@ def truediv(t1: Tensor, t2: Tensor) -> Tensor:
     dependencies: List[Dependency] = []
 
     if t1.requires_grad:
-        def grad_fn1(grad: np.ndarray) -> np.ndarray:
+        def grad_fn1(grad: np.ndarray, _) -> np.ndarray:
             grad = np.divide(grad, t2.data)
             ndims_added = grad.ndim - t1.data.ndim
             for _ in range(ndims_added):
@@ -379,13 +475,13 @@ def truediv(t1: Tensor, t2: Tensor) -> Tensor:
                     grad = grad.sum(axis=i, keepdims=True)
             return grad
 
-        dependencies.append(Dependency(t1, grad_fn1))
+        dependencies.append(Dependency(t1, grad_fn1, meta={"name": "truediv_lhs"}))
 
     if t2.requires_grad:
-        def grad_fn2(grad: np.ndarray) -> np.ndarray:
+        def grad_fn2(grad: np.ndarray, _) -> np.ndarray:
             grad = - (grad / t2.data) * (t1.data / t2.data)
-            # grad = np.negative(np.divide(np.multiply(grad, t1.data), t2.data * t2.data))
-            # grad = np.negative(np.divide(np.multiply(grad, t1.data), np.power(t2.data, 2)))
+            # grad = np.negative(np.divide(np.multiply(grad, t1.datasets), t2.datasets * t2.datasets))
+            # grad = np.negative(np.divide(np.multiply(grad, t1.datasets), np.power(t2.datasets, 2)))
             ndims_added = grad.ndim - t2.data.ndim
             for _ in range(ndims_added):
                 grad = grad.sum(axis=0)
@@ -394,12 +490,13 @@ def truediv(t1: Tensor, t2: Tensor) -> Tensor:
                     grad = grad.sum(axis=i, keepdims=True)
             return grad
 
-        dependencies.append(Dependency(t2, grad_fn2))
+        dependencies.append(Dependency(t2, grad_fn2, meta={"name": "truediv_rhs"}))
 
     return Tensor(data, requires_grad, dependencies)
 
 
 def matmul(t1: Tensor, t2: Tensor) -> Tensor:
+    # TODO: fix one dim
     """
     if t1 is (n1, m1) and t2 is (m1, m2), then t1 @ t2 is (n1, m2)
     so grad3 is (n1, m2)
@@ -412,16 +509,30 @@ def matmul(t1: Tensor, t2: Tensor) -> Tensor:
     dependencies: List[Dependency] = []
 
     if t1.requires_grad:
-        def grad_fn1(grad: np.ndarray) -> np.ndarray:
-            return grad @ np.swapaxes(t2.data, -1, -2)
+        def grad_fn1(grad: np.ndarray, _) -> np.ndarray:
+            grad = grad @ np.swapaxes(t2.data, -1, -2)
+            ndims_added = grad.ndim - t1.data.ndim
+            for _ in range(ndims_added):
+                grad = grad.sum(axis=0)
+            for i, dim in enumerate(t1.shape):
+                if dim == 1:
+                    grad = grad.sum(axis=i, keepdims=True)
+            return grad
 
-        dependencies.append(Dependency(t1, grad_fn1))
+        dependencies.append(Dependency(t1, grad_fn1, meta={"name": "matmul_lhs"}))
 
     if t2.requires_grad:
-        def grad_fn2(grad: np.ndarray) -> np.ndarray:
-            return np.swapaxes(t1.data, -1, -2) @ grad
+        def grad_fn2(grad: np.ndarray, _) -> np.ndarray:
+            grad = np.swapaxes(t1.data, -1, -2) @ grad
+            ndims_added = grad.ndim - t2.data.ndim
+            for _ in range(ndims_added):
+                grad = grad.sum(axis=0)
+            for i, dim in enumerate(t2.shape):
+                if dim == 1:
+                    grad = grad.sum(axis=i, keepdims=True)
+            return grad
 
-        dependencies.append(Dependency(t2, grad_fn2))
+        dependencies.append(Dependency(t2, grad_fn2, meta={"name": "matmul_rhs"}))
 
     return Tensor(data, requires_grad, dependencies)
 
@@ -433,7 +544,7 @@ def power(x: Tensor, e: Tensor) -> Tensor:
     dependencies: List[Dependency] = []
 
     if x.requires_grad:
-        def grad_fn1(grad: np.ndarray) -> np.ndarray:
+        def grad_fn1(grad: np.ndarray, _) -> np.ndarray:
             grad = grad * e.data * np.power(x.data, e.data - 1)
             ndims_added = grad.ndim - x.data.ndim
             for _ in range(ndims_added):
@@ -443,10 +554,10 @@ def power(x: Tensor, e: Tensor) -> Tensor:
                     grad = grad.sum(axis=i, keepdims=True)
             return grad
 
-        dependencies.append(Dependency(x, grad_fn1))
+        dependencies.append(Dependency(x, grad_fn1, meta={"name": "power_lhs"}))
 
     if e.requires_grad:
-        def grad_fn2(grad: np.ndarray) -> np.ndarray:
+        def grad_fn2(grad: np.ndarray, _) -> np.ndarray:
             grad = grad * data * np.log(x.data)
             ndims_added = grad.ndim - e.data.ndim
             for _ in range(ndims_added):
@@ -456,14 +567,19 @@ def power(x: Tensor, e: Tensor) -> Tensor:
                     grad = grad.sum(axis=i, keepdims=True)
             return grad
 
-        dependencies.append(Dependency(e, grad_fn2))
+        dependencies.append(Dependency(e, grad_fn2, meta={"name": "power_rhs"}))
 
     return Tensor(data, requires_grad, dependencies)
 
 
 def dot(t1: Tensor, t2: Tensor) -> Tensor:
     # TODO: impl, test
-    pass
+    assert t1.dim() <= 1 and t2.dim() <= 1
+    data = np.dot(t1.data, t2.data)
+    requires_grad = t1.requires_grad or t2.requires_grad
+    dependencies: List[Dependency] = []
+
+    return Tensor(data, requires_grad, dependencies)
 
 
 def transpose(a: Tensor, dim1: int, dim2: int) -> Tensor:
@@ -472,10 +588,10 @@ def transpose(a: Tensor, dim1: int, dim2: int) -> Tensor:
     dependencies: List[Dependency] = []
 
     if a.requires_grad:
-        def grad_fn(grad: np.ndarray) -> np.ndarray:
+        def grad_fn(grad: np.ndarray, _) -> np.ndarray:
             return np.swapaxes(grad, dim2, dim1)
 
-        dependencies.append(Dependency(a, grad_fn))
+        dependencies.append(Dependency(a, grad_fn, meta={"name": "transpose"}))
 
     return Tensor(data, requires_grad, dependencies)
 
@@ -486,19 +602,10 @@ def permute(a: Tensor, axes: Union[List, Tuple]) -> Tensor:
     dependencies: List[Dependency] = []
 
     if a.requires_grad:
-        def grad_fn(grad: np.ndarray) -> np.ndarray:
+        def grad_fn(grad: np.ndarray, _) -> np.ndarray:
             axes_t = invert_permutation(axes)
             return grad.transpose(axes_t)
 
-        dependencies.append(Dependency(a, grad_fn))
-
-    return Tensor(data, requires_grad, dependencies)
-
-
-def concat(tensors: List[Tensor], axis=0) -> Tensor:
-    # TODO: impl
-    data = np.concatenate([i.data for i in tensors], axis=axis)
-    requires_grad = True in [i.requires_grad for i in tensors]
-    dependencies: List[Dependency] = []
+        dependencies.append(Dependency(a, grad_fn, meta={"name": "permute"}))
 
     return Tensor(data, requires_grad, dependencies)
