@@ -7,6 +7,15 @@ from .activation import (
 from .container import (
     Sequential
 )
+from .conv import (
+    Conv2d
+)
+from .dropout import (
+    Dropout
+)
+from .flatten import (
+    Flatten
+)
 from .linear import (
     Linear
 )
@@ -18,3 +27,6 @@ from .loss import (
 from .module import (
     Module
 )
+from .pooling import (
+    MaxPool2d
+)
